import sys, os, datetime, optparse, inspect
import numpy
from atmopy import *
from atmopy.display import *

import scipy

from pylab import *
from matplotlib.dates import WeekdayLocator, DayLocator, DateFormatter

try:
    from matplotlib.toolkits.basemap import Basemap, shiftgrid
except:
    pass
# From version 0.98.0 of Matplotlib.
try:
    from mpl_toolkits.basemap import Basemap, shiftgrid
except:
    pass

from matplotlib.font_manager import *


### function: read_stations_Chernobyl

def read_stations_chernobyl(filename):
    """
    Loads stations for the Chernobyl case from a text file.

    @type filename: string
    @param filename: The text file in which station
    attributes are to be found.
    @rtype: Station sequence
    @return: Stations sequence
    """

    stations = []
    f = open(filename)

    for line in f.readlines():
        i = line.split()

        name = str(i[3])
        longitude = float(i[1])
        latitude = float(i[2])

        station = observation.Station()
        station.name = name
        station.latitude = latitude
        station.longitude = longitude

        stations.append(station)

    f.close()

    return stations


### function: read measures chernobyl

def read_measurements_chernobyl(config, stations):
    """
    Reads measurements.
    """
    mes = [[] for i in range(len(stations))]
    date = [[] for i in range(len(stations))]

    for k in range(len(stations)):

        filename = os.path.normpath(config.obs_dir) + '/' \
                   + stations[k].name  + ".dat"

        f = open(filename)
        for j in f.readlines():
            line = j.split()
            date_read = line[0]
            year = int("19" + date_read[0:2])
            month = int(date_read[3:5])
            day = int(date_read[6:])

            begin = line[1].split(":")
            hour_begin = int(begin[0])
            if len(begin) > 1:
                min_begin = int(begin[1])
            else:
                min_begin = 0

            delta_time = datetime.timedelta(hours=hour_begin, minutes=min_begin)
            start_date = datetime.datetime(year, month, day) - delta_time

            period = line[2].split(".")
            hour_period = int(period[0])
            if len(period) > 1:
                min_period = int(period[1])
            else:
                min_period = 0
            duration = datetime.timedelta(hours=hour_period, minutes=min_period)

            # Accumulated obs
            end_date = start_date + duration

            # which species?
            n_species = -1
            species = ["I131", "Cs134", "Cs137"]
            for i in range(len(species)):
                if config.species == species[i]:
                    n_species = i

            if n_species < 0:
                raise Exception, "unknown specie: " + config.species + "\t" \
                      + species[-1]

            observ  = line[3 + n_species]
            if not talos.is_num(observ):
                observ = -9999

            mes[k].append(float(observ))
            date[k].append((start_date, end_date))

        f.close()

        obs = [[] for i in range(len(mes))]
        for i in range(len(mes)):
            for j in range(len(mes[i])):
                obs[i].append((i, numpy.array(mes[i][j], 'f'), date[i][j]))


    return obs


############################
def get_averaged_concentrations(date, sim_dates, sim):

    simul = [[] for i in range(len(date))]
    for i in range(len(date)):
        for j in range(len(date[i])):

            start_obs = date[i][j][0]
            end_obs = date[i][j][1]

            t = 0
            while sim_dates[t] <= start_obs:
                t += 1
            istart = t - 1
            while sim_dates[t] < end_obs:
                t += 1
            ifinish = t

            dt_l = (start_obs - sim_dates[istart]).seconds

            A_l = ( sim[i][istart] + 0.5 * dt_l \
                        / (sim_dates[istart+1] - sim_dates[istart]).seconds \
                        * (sim[i][istart+1] - sim[i][istart])) * dt_l

            A_m = 0
            for l in range(istart, ifinish):
                A_m += 0.5 * (sim[i][l] + sim[i][l+1]) \
                    * (sim_dates[l+1] - sim_dates[l]).seconds

            dt_r = (sim_dates[ifinish] - end_obs).seconds

            A_r = ( sim[i][ifinish] - 0.5 * dt_r \
                        / (sim_dates[ifinish] - sim_dates[ifinish-1]).seconds \
                        * (sim[i][ifinish] - sim[i][ifinish-1])) * dt_r

            period = sim_dates[ifinish] - sim_dates[istart]
            duration = period.days * 24. * 3600. + period.seconds

            A = array((A_m - A_r - A_l) / (duration - dt_r - dt_l), 'f')
            if isnan(A):
                print(A, duration, dt_r, dt_l, i)
            simul[i].append(A)

    return simul


############################
def filter_obs(observ_ref, sim_dates, d_station, stations, meas_min):

    observ = []
    for i in range(len(observ_ref)):
        observ.append(observ_ref[i])

    for i in range(len(observ)):
        A = []
        for j in range(len(observ[i])):

            start_obs = observ[i][j][2][0]
            end_obs = observ[i][j][2][1]
            start_sim = sim_dates[0]
            end_sim = sim_dates[-1]
            obs_value = observ[i][j][1]
            number = observ[i][j][0]

            if start_obs < start_sim or end_sim < end_obs \
                   or float(obs_value) < 0:
                A.append((number, obs_value, (start_obs, end_obs)))

        for a in A:
            observ[i].remove(a)

    for i in range(observ.count([])):
        observ.remove([])

    if len(observ) == 0:
        raise Exception, "No observations found in the simulation period."

    size = 0
    for i in range(len(observ)):
        if len(observ[i]) >= meas_min: size += 1

    obs_dates = [[] for i in range(size)]
    obs = [[] for i in range(size)]
    stations_filter = []
    d_station_filter = []
    size = 0
    for i in range(len(observ)):
        if len(observ[i]) >= meas_min:
            num = observ[i][0][0]
            for j in range(len(observ[i])):
                obs_dates[size].append(observ[i][j][2])
                obs[size].append(observ[i][j][1])
            d_station_filter.append(d_station[num])
            stations_filter.append(stations[num])
            size += 1

    return obs_dates, obs, d_station_filter, stations_filter

############################
def compute_observation_bar(obs_dates):

    bar_ind = [[] for i in range(len(obs_dates))]
    bar_width = [[] for i in range(len(obs_dates))]
    for i in range(len(obs_dates)):
        for j in range(len(obs_dates[i])):
            d0 = date2num(obs_dates[i][j][0])
            d1 = date2num(obs_dates[i][j][1])
            bar_ind[i].append(d0)
            bar_width[i].append(d1 - d0)

    return bar_ind, bar_width

############################
def get_map(config):

    m = Basemap(projection = 'cyl',
                llcrnrlon = config.x_min - config.Delta_x / 2.,
                llcrnrlat = config.y_min - config.Delta_y / 2.,
                urcrnrlon = config.x_min + config.Delta_x / 2. +
                config.Delta_x * float(config.Nx),
                urcrnrlat = config.y_min + config.Delta_y / 2. +
                config.Delta_y * float(config.Ny), resolution = 'l')
    axes([0.1, 0.15, 0.8, 0.75])

    m.drawcountries()
    m.drawcoastlines()

    return m

############################
def get_surface_coordinates(config):

    x = config.x_min + arange(config.Nx) * config.Delta_x
    y = config.y_min + arange(config.Ny) * config.Delta_y
    xx = array([x for i in y])
    yy = array([y for i in x])
    yy = yy.transpose()

    return xx, yy

############################
def axes_label(levels, palette):

    labels=["%2.E" % i for i in levels[1:-1]]
    #alette = palette[:len(levels)]
    cax = axes([0.3, 0.05, 0.4, 0.02])
    setp(cax, xticks=[], yticks=[])
    for i in range(len(palette)):
        cax.fill([i,i+1,i+1,i],[0,0,1,1],palette[i])
    for i in range(len(labels)):
        text(i+1, -1., labels[i], horizontalalignment='center', \
             verticalalignment='center', fontsize=8, color='k')

    return cax

